'Motion detector with LDR or BPW34
'
'running average to establish Avglight, threshold and trigger
'Ledalarm turned on 100 ms
'Lednight turned on when low light
'10 ms sampling time
'
'+5V -LDR- PB3 -10k- -GND
'
'***************************************************************************
'
'  Pinout ATtiny13/ATtiny13V   8-PDIP/SOIC
'
' (PCINT5/RESET/ADC0/dW)   PB5 VCC
' (PCINT3/CLKI/ADC3)       PB3 PB2 (SCK/ADC1/T0/PCINT2)
' (PCINT4/ADC2)            PB4 PB1 (MISO/AIN1/OC0B/INT0/PCINT1)
'                          GND PB0 (MOSI/AIN0/OC0A/PCINT0)
'
'***************************************************************************
' PB0 output  Ipulse (calibrate 100 Hz)
' PB1 output  Ledalarm hi
' PB2 output  Lednight hi
' PB4 output  COM1
' PB3 input   ADC channel 3 LDR or BPW34
'***************************************************************************

$regfile = "attiny13.dat"
$crystal = 1200000
$hwstack = 8
$swstack = 8
$framesize = 4

Const C_value = 189 - 1                                     ' Compare Match IRQ 10 ms, 53 x 189
' C_value = 94 for 5 ms w Presc 64
' C_value = 150 for 1 ms w Presc 8

Const Threshval = 32                                        'Avglight divide by Threshval
Const Lolight = 40                                          'turn on Lednight
Const Hilight = 60                                          'turn off Lednight

Ipulse Alias Portb.0
Ledalarm Alias Portb.1
Lednight Alias Portb.2

Dim Currentlight As Word , Avglight As Word
Dim Thresh As Byte , Trig As Word
Dim Icnt As Byte , Dcnt As Byte , Alarmflag As Byte

Portb = &B00000000
Ddrb = &B00010111

Config Adc = Single , Prescaler = Auto , Reference = Avcc
Didr0.adc3d = 1
Start Adc

Config Timer0 = Timer , Prescale = 64
Ocr0a = C_value                                             ' value for 1.2 MHz
Tccr0a = &B00000010                                         ' ctc
Timsk0.ocie0a = 1
On Oc0a Tim0_compa
Start Timer0

Acsr.acd = 0                                                ' switch off analog comparator

Open "comb.4:9600,8,n,1,INVERTED" For Output As #1
Enable Interrupts

Avglight = 10

Do
  If Icnt = 100 Then                                        '1000 ms
   Icnt = 0
   Print #1 , Avglight
  End If
Loop                                                        'main loop

'***************************************************************************
'Tim0_compa Interrupt  10 ms   IRQ needs 600us
'***************************************************************************

Tim0_compa:                                                 '10 ms
  Ipulse = 1
  Incr Icnt

  Currentlight = Getadc(3)
  Avglight = Avglight * 15                                  ' (Avglight*15 + Currentlight)/16
  Avglight = Avglight + Currentlight
  Shift Avglight , Right , 4                                'running average

  Thresh = Avglight / Threshval                             'Threshold 3.1 %
  Trig = Avglight - Thresh
  If Currentlight < Trig Then
    Alarmflag = 1                                           'Alarm on
  End If

  If Avglight < Lolight Then                                'when low light turn on Led
    Lednight = 1
    Ledalarm = 0                                            ' Ledalarm off
    Alarmflag = 0                                           'no alarm when dark
  Elseif Avglight > Hilight Then                            'turn off Led
    Lednight = 0
  End If

  If Alarmflag = 1 Then                                     'sw monoflop 100 ms
    Ledalarm = 1                                            ' Led on
    Incr Dcnt
    If Dcnt = 10 Then                                       'reset sw monoflop
       Ledalarm = 0
       Alarmflag = 0
       Dcnt = 0
    End If
  End If                                                    'if Alarmflag = 1
  Ipulse = 0

Return


End